//
//  DCManager.m
//  Debug-Controller
//
//  Created by Tom Susel on 2/5/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "DCManager.h"

#define degreesToRadian(x) (M_PI * (x) / 180.0)

@interface DCManager ()

- (void)rotateViewAccordingToOrientation:(UIView*)targetView;

@end

@implementation DCManager
@synthesize debugActions = _debugActions;

static DCManager *sharedInstance = nil;

#pragma mark -
#pragma mark class instance methods

#pragma mark -
#pragma mark Singleton methods

+ (DCManager*)sharedInstance
{
    @synchronized(self)
    {
        if (sharedInstance == nil)
            sharedInstance = [[DCManager alloc] init];
    }
    return sharedInstance;
}

+ (id)allocWithZone:(NSZone *)zone {
    @synchronized(self) {
        if (sharedInstance == nil) {
            sharedInstance = [super allocWithZone:zone];
            return sharedInstance;  // assignment and return on first allocation
        }
    }
    return nil; // on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain {
    return self;
}

- (NSUInteger)retainCount {
    return UINT_MAX;  // denotes an object that cannot be released
}

- (void)release {
    //do nothing
}

- (id)autorelease {
    return self;
}

#pragma mark -
#pragma mark NSObject

- (id)init{
	if (self = [super init]){
		_debugActions = [[NSMutableArray alloc] initWithCapacity:5];
		_viewController = [[DCViewController alloc] init];
		_viewController.view;
		
		_shakeRecognizer = [[DCShakeRecognizer alloc] init];
	}
	
	return self;
}


#pragma mark -
#pragma mark DCManager - public

- (void)addAction:(DCDebugAction*)debugAction{
	if (debugAction != nil){
		[_debugActions addObject:debugAction];
	}
}

- (void)addActionWithTitle:(NSString*)title
					target:(id)target
				  selector:(SEL)aSelector
					object:(NSObject*)anObject{
	DCDebugAction *debugAction = [[DCDebugAction alloc] initWithTitle:title
															   target:target
															 selector:aSelector
															   object:anObject];
	
	[self addAction:debugAction];
	[debugAction release];
}

- (void)showController{
	if (_viewController.view.superview == nil){
		UIWindow *window = [[UIApplication sharedApplication] keyWindow];
		
		_viewController.view.center = window.center;
		
		[_viewController viewWillAppear:NO];
		[window addSubview:_viewController.view];
		
		[self rotateViewAccordingToOrientation:_viewController.view];
		
		[_viewController viewDidAppear:NO];		
	}
}

- (void)hideController{
	[_viewController viewWillDisappear:NO];
	[_viewController.view removeFromSuperview];
	[_viewController viewDidDisappear:NO];
}

- (void)toggleController{
	if (_viewController.view.superview != nil){
		[self hideController];
	}
	else {
		[self showController];
	}

}

#pragma mark -
#pragma mark DCManager - private

- (void)rotateViewAccordingToOrientation:(UIView*)targetView{
	UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
	
	CGFloat degrees = 90.0;
	
	switch (orientation) {
		case UIInterfaceOrientationPortrait:
			degrees = 0.0;
			break;
		case UIInterfaceOrientationPortraitUpsideDown:
			degrees *= 2.0;
			break;
		case UIInterfaceOrientationLandscapeRight:
			break;
		case UIInterfaceOrientationLandscapeLeft:
			degrees *= 3.0;
			break;
		default:
			break;
	}
	_viewController.view.transform = CGAffineTransformRotate(CGAffineTransformIdentity,
															 degreesToRadian(degrees));
	
}

@end
