//
//  DCShakeRecognizer.m
//  Debug-Controller
//
//  Created by Tom Susel on 2/5/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "DCShakeRecognizer.h"
#import "DCManager.h"

#define kShakeRecognizerThreshold 0.4
#define kShakeRecognizerUpdateFrequence 40.0
#define kShakeRecognizerMinimumDelayBetweenShakes 2.0

@interface DCShakeRecognizer ()

- (BOOL)isShaking:(UIAcceleration*)last current:(UIAcceleration*)current;

@end

@implementation DCShakeRecognizer

#pragma mark -
#pragma mark NSObject

- (id)init{
	if ((self = [super init])){
		_lastAcceleration = nil;
		
		//Initiate accelerometer observing
		[UIAccelerometer sharedAccelerometer].delegate = self;
		[[UIAccelerometer sharedAccelerometer] setUpdateInterval:(20.0 / kShakeRecognizerUpdateFrequence)];
	}
	return self;
}

- (void)dealloc{
	[UIAccelerometer sharedAccelerometer].delegate = nil;
	
	[_lastAcceleration release];
	_lastAcceleration = nil;
	
	
	[super dealloc];
}
#pragma mark -
#pragma mark DCShakeRecognizer - public

#pragma mark -
#pragma mark DCShakeRecognizer - private

- (BOOL)isShaking:(UIAcceleration*)last current:(UIAcceleration*)current{
	double
	deltaX = fabs(last.x - current.x),
	deltaY = fabs(last.y - current.y),
	deltaZ = fabs(last.z - current.z);
	
	return
	(deltaX > kShakeRecognizerThreshold && deltaY > kShakeRecognizerThreshold) ||
	(deltaX > kShakeRecognizerThreshold && deltaZ > kShakeRecognizerThreshold) ||
	(deltaY > kShakeRecognizerThreshold && deltaZ > kShakeRecognizerThreshold);	
}

#pragma mark -
#pragma mark UIAccelerometerDelegate


- (void) accelerometer:(UIAccelerometer *)accelerometer didAccelerate:(UIAcceleration *)acceleration{
	static NSTimeInterval lastShakeTime = 0.0;
    
    NSTimeInterval timeSinceLastShake = [[NSDate date] timeIntervalSinceReferenceDate] - lastShakeTime;
    
	if (_lastAcceleration && 
        timeSinceLastShake > kShakeRecognizerMinimumDelayBetweenShakes ) {
        
		if ([self isShaking:_lastAcceleration current:acceleration] == YES){
			[[DCManager sharedInstance] toggleController];
            lastShakeTime = [[NSDate date] timeIntervalSinceReferenceDate];
		}
	}
	
	[_lastAcceleration release];
	_lastAcceleration = [acceleration retain];
}
@end
